@@ -22,6 +22,7 @@
 
 #include <bonobo/bonobo-widget.h>
 #include <bonobo/bonobo-ui-xml.h>
+#include <bonobo/bonobo-i18n.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-ui-container.h>
 #include <bonobo/bonobo-ui-private.h>
@@ -72,6 +73,26 @@ enum {
 
 static guint signals [LAST_SIGNAL] = { 0 };
 
+static void
+add_debug_menu (BonoboUIEngine *engine)
+{
+	char *xml;
+	BonoboUINode *node;
+
+	xml = g_strdup_printf (
+		"<menu>"
+		"  <submenu name=\"BonoboDebug\" label=\"%s\">"
+		"      <menuitem name=\"BonoboUIDump\" verb=\"\""
+		"       label=\"%s\" tip=\"%s\"/>"
+		"  </submenu>"
+		"</menu>", _("Debug"), _("_Dump XML"),
+		_("Dump the entire UI's XML description to the console"));
+
+	node = bonobo_ui_node_from_string (xml);
+	bonobo_ui_engine_xml_merge_tree (engine, "/", node, "BuiltIn");
+	g_free (xml);
+}
+
 /*
  *  Mapping from nodes to their synchronization
  * class information functions.
@@ -1173,8 +1194,9 @@ bonobo_ui_engine_widget_set (BonoboUIEngine    *engine,
 			     GtkWidget         *widget)
 {
 	NodeInfo *info;
-	GtkWidget *tool_item;
+	GtkWidget *custom_widget;
 	BonoboUINode *node;
+	BonoboUISync *sync;
 
 	g_return_if_fail (widget != NULL);
 
@@ -1187,16 +1209,17 @@ bonobo_ui_engine_widget_set (BonoboUIEngine    *engine,
 	g_return_if_fail (node != NULL);
 	g_return_if_fail (!strcmp (bonobo_ui_node_get_name (node), "control"));
 
-	if (BONOBO_IS_UI_TOOLBAR_ITEM (widget))
-		tool_item = widget;
-	else
-		tool_item = bonobo_ui_toolbar_control_item_new_widget (widget);
+	sync = find_sync_for_node (engine, node);
 
-	info = bonobo_ui_xml_get_data (engine->priv->tree, node);
-	info->widget = gtk_widget_ref (tool_item);
-	gtk_object_sink (GTK_OBJECT (tool_item));
+	custom_widget = bonobo_ui_sync_wrap_widget (sync, widget);
 
-	bonobo_ui_engine_stamp_custom (engine, node);
+	if (custom_widget) {
+		info = bonobo_ui_xml_get_data (engine->priv->tree, node);
+		info->widget = gtk_widget_ref (custom_widget);
+		gtk_object_sink (GTK_OBJECT (custom_widget));
+
+		bonobo_ui_engine_stamp_custom (engine, node);
+	}
 		
 	bonobo_ui_engine_thaw (engine);
 }
@@ -2020,6 +2043,9 @@ bonobo_ui_engine_construct (BonoboUIEngine *engine,
 
 	build_skeleton (priv->tree);
 
+	if (g_getenv ("BONOBO_DEBUG"))
+		add_debug_menu (engine);
+
 	g_signal_connect (priv->tree, "override",
 			  (GtkSignalFunc) override_fn, engine);
 
